/* **********
 *
 * Test Program
 *
 * Copyright (c) 2003 Dan B Goldman, Aaron Hertzmann
 * 
 * This file is part of the BossWorker library.
 *
 * BossWorker is free software; you can redistribute it and/or modify
 * it under the terms of the X11 Software License (see the LICENSE file
 * for details).
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the X11
 * License for more details.
 *
 * See BossWorker.h for API documentation
 *
 * **********/

#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include "BossWorker.h"

using namespace std;

bool boss = false;
int numJobs = 10;
char *bossAddress = "localhost";
int bossPort = 1234;

void
parseArgs(int argc, char **argv)
{
    char **arg;

    for (arg = argv+1, argc--; argc--; arg++) {
	if (strcmp(*arg,"-b") == 0) {
	    boss = true;
	    bossPort = atoi(*(++arg));
	    argc--;
	} else if (strcmp(*arg,"-w") == 0) {
	    boss = false;
	    bossAddress = *(++arg);
	    bossPort = atoi(*(++arg));
	    argc -= 2;
	} else if (strcmp(*arg,"-j") == 0) {
	    numJobs = atoi(*(++arg));
	    argc--;
	} else 
	    printf("unknown argument \"%s\"\n",*arg);
    }

    return;
}

class MyJob : public JobMessage {
public:
  int i;
  inline MyJob() {i=0;}
  inline MyJob(int mi) : i(mi) {};
  inline string pack() const {return string((char*)&i,sizeof(i));}
  inline void unpack(const string &s) {i = *(int *)s.data();}
};

class MyResult : public ResultMessage {
public:
  int i;
  inline MyResult() {i=0;}
  inline MyResult(int mi) : i(mi) {};
  inline string pack() const {return string((char*)&i,sizeof(i));}
  inline void unpack(const string &s) {i = *(int *)s.data();}
};
  
main(int argc, char ** argv)
{
    parseArgs(argc,argv);

    if (boss) {
	Boss theBoss(bossPort);

	int jobsLeft;
	for (jobsLeft = numJobs; jobsLeft; jobsLeft--) {
	    MyJob job(numJobs - jobsLeft);
	    theBoss.send(job);
	    printf("boss sent \"%d\"\n",job.i);
	}
	for (jobsLeft = numJobs; jobsLeft; jobsLeft--) {
	    //	  string results;
	    MyResult results;
	    theBoss.receive(results);
	    printf("boss received \"%d\"\n",results.i);
	}

	printf("Boss done\n");
    } else {
	Worker theWorker(bossAddress, bossPort);

	struct timeval tv;
	gettimeofday(&tv,NULL);
	srandom(tv.tv_sec ^ tv.tv_usec);

	MyJob job;

	while (theWorker.receive(job)) {
	    printf("worker received \"%d\"\n",job.i);

/*
	    int stime = random()%100000 + 1;  // random amount under 1 second
	    printf("sleeping for %d usecs\n",stime);
	    usleep(stime);
*/

	    int stime = random()%3 + 1; // random amount from 1 to 3 seconds
	    printf("sleeping for %d secs\n",stime);
	    sleep(stime);

	    MyResult results(job.i*2);
	    theWorker.send(results);
	    printf("worker sent \"%d\"\n",results.i);
	}
	       
	printf("Worker done\n");
    }

}
